#!/bin/bash

# Usage info
show_help() {
cat << EOF
Usage: ${0##*/} [-h] [-m MEMORY] [XPIPFILE]...
Launch X!TandemPipeline on XPIPFILE. With no XPIPFILE, launches X!TandemPipeline with a new empty project.

    -h          display this help and exit
    -m MEMORY   the required memory in Mbytes (3000 for 3Gb)
EOF
}

# Initialize our own variables:
memory="1500"
jvm="/usr/bin/java"
jarfile="@CMAKE_INSTALL_PREFIX@/share/xtandempipeline/XtandemPipeline.jar"

if [ $(uname -m) == "x86_64" ]; then
	memory="3000"
	if [ -f "/usr/lib/jvm/java-6-openjdk-amd64/bin/java" ]; then
		jvm="/usr/lib/jvm/java-6-openjdk-amd64/bin/java"
	else
		if [ -f "/usr/lib/jvm/java-7-openjdk-amd64/bin/java" ]; then
			jvm="/usr/lib/jvm/java-7-openjdk-amd64/bin/java"
		fi
	fi
else
	memory="1500"
	if [ -f "/usr/lib/jvm/java-6-openjdk-i386/bin/java" ]; then
		jvm="/usr/lib/jvm/java-6-openjdk-i386/bin/java"
	else
		if [ -f "/usr/lib/jvm/java-7-openjdk-i386/bin/java" ]; then
			jvm="/usr/lib/jvm/java-7-openjdk-i386/bin/java"
		fi
	fi
fi

OPTIND=1 # Reset is necessary if getopts was used previously in the script.  It is a good idea to make this local in a function.
while getopts "hm:" opt; do
    case "$opt" in
        h)
            show_help
            exit 0
            ;;
        m)  memory=$OPTARG
            ;;
        '?')
            show_help >&2
            exit 1
            ;;
    esac
done
shift "$((OPTIND-1))" # Shift off the options and optional --.

#printf 'java -Xmx%sm -Xms256m \n' "$memory"
#printf '<%s>\n' "$@"

$jvm -Xmx${memory}m -Xms256m -jar $jarfile $@

# End of file
