/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class PeptideMassSet extends TreeSet<PeptideMass> {
	private static final Logger logger = Logger.getLogger(PeptideMassSet.class);

	// private HashMap<PeptideMass, PeptideMass> pm2pm = new
	// HashMap<PeptideMass, PeptideMass>(
	// 0);
	private static final long serialVersionUID = -5367891674312454987L;

	private Hashtable<String, PeptideMass> pepSampleScanCle2peptideMassTmp = null;

	private HashMap<PeptideMass, HashSet<Peptide>> pepMass2PeptideListTmp = null;

	private boolean modePhospho = false;

	private boolean ignoreIsotopeLabels = true;

	// build a peptide/mass set
	public PeptideMassSet() {
		// the default behaviour is to consider mass shift of the same peptide
		// sequence as different peptides
		// but if isotope label were defined, different labels of the same
		// peptide (different mass shifts) are considered as the same peptide
		ignoreIsotopeLabels = true;
	}

	public PeptideMass getInstance(Peptide peptide) throws MSMSException {
		boolean removeMS3 = true;
		PeptideMass pepMass = new PeptideMass(peptide.get_LI_sequence(),
				peptide.getMassShift(ignoreIsotopeLabels, removeMS3));
		if (!this.contains(pepMass)) {
			String message = "no PeptideMass corresonding to this peptide";
			logger.error(message);
			throw new MSMSException(message);
		}
		return this.getInstance(pepMass);
	}

	public PeptideMass addPeptide(Peptide peptide) throws MSMSException {
		boolean removeMS3 = true;
		PeptideMass pepMass = new PeptideMass(peptide.get_LI_sequence(),
				peptide.getMassShift(ignoreIsotopeLabels, removeMS3));

		logger.debug(peptide.get_LI_sequence() + " "
				+ peptide.get_mass_Modifs_without_isotope(true).doubleValue());
		if (this.contains(pepMass)) {
			pepMass = this.getInstance(pepMass);
		} else {
			// pm2pm.put(pepMass, pepMass);
			this.add(pepMass);
		}
		pepMass.addPeptide(peptide);
		return pepMass;
	}

	private PeptideMass getInstance(PeptideMass pepMass) throws MSMSException {
		PeptideMass pepMassRet = this.ceiling(pepMass);
		if (pepMassRet.equals(pepMass)) {
			return pepMassRet;
		} else {
			String message = "pepMassRet (" + pepMassRet.getSequenceLi()
					+ ") != pepMass" + pepMass.getSequenceLi() + " ";
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void numbering() {
		int i = 1;
		for (PeptideMass pepMass : this) {
			pepMass.setNumber(i);
			i++;
			// pm2pm.put(pepMass, pepMass);
		}
	}

	public Hashtable<String, PeptideMass> numberingGetSampleScanCle2peptideMass(
			Group group) throws MSMSException {
		Hashtable<String, PeptideMass> pepSampleScanCle2peptideMass = new Hashtable<String, PeptideMass>();
		this.pepSampleScanCle2peptideMassTmp = pepSampleScanCle2peptideMass;

		this.numbering(group);
		this.pepSampleScanCle2peptideMassTmp = null;
		return pepSampleScanCle2peptideMass;
	}

	public HashMap<PeptideMass, HashSet<Peptide>> numberingGetPepMass2PeptideList(
			Group group) throws MSMSException {

		HashMap<PeptideMass, HashSet<Peptide>> pepMass2PeptideList = new HashMap<PeptideMass, HashSet<Peptide>>();
		pepMass2PeptideListTmp = pepMass2PeptideList;
		this.numbering(group);
		pepMass2PeptideListTmp = null;
		return pepMass2PeptideList;
	}

	public void numbering(Group group) throws MSMSException {

		for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

			for (HashSampleScanSetProt hashProt : sg
					.getHashSampleScanSetProtList()) {
				if (modePhospho) {
					this.forEachPeptide(
							((Match) hashProt.getGrpProtMatch())
									.getValidPeptideInPhosphoIsland((HashSampleScanSetPhospho) hashProt),
							sg);

				} else {
					this.forEachPeptide(((Match) hashProt.getGrpProtMatch())
							.get_valid_peptides(), sg);

				}

			}
		}

		this.numbering();
	}

	private void forEachPeptide(ArrayList<Peptide> peptideListLoop, SubGroup sg)
			throws MSMSException {
		for (Peptide peptide : peptideListLoop) {
			// first pass to index each ScanSample and give it a
			// PeptideMass
			if (peptide.isValid()) {
				PeptideMass pepMass = this.addPeptide(peptide);
				pepMass.addSubGroup(sg);

				if (pepMass2PeptideListTmp != null) {
					HashSet<Peptide> peptideList;
					if (pepMass2PeptideListTmp.containsKey(pepMass)) {
						peptideList = pepMass2PeptideListTmp.get(pepMass);
					} else {
						peptideList = new HashSet<Peptide>();
						pepMass2PeptideListTmp.put(pepMass, peptideList);
					}
					peptideList.add(peptide);
				} else if (pepSampleScanCle2peptideMassTmp != null) {
					pepSampleScanCle2peptideMassTmp.put(
							peptide.get_sample_scan_sequencepeptide_cle(),
							pepMass);
				}
			} else {
				String message = "peptide is not valid";
				logger.error(message);
				throw new MSMSException(message);
			}
		}
	}

	public HashMap<PeptideMass, HashSet<Peptide>> numberingPhospho(Group group)
			throws MSMSException {

		modePhospho = true;

		return numberingGetPepMass2PeptideList(group);
	}

	public Hashtable<String, PeptideMass> numberingPhosphoGetSampleScanCle2peptideMass(
			Group group) throws MSMSException {
		modePhospho = true;

		return numberingGetSampleScanCle2peptideMass(group);
	}
}
