package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import java.io.File;

import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;

public abstract class IdentificationDataFile implements
		IdentificationDataSource {
	private static final Logger logger = Logger
			.getLogger(IdentificationDataFile.class);

	protected File file = null;
	private String fileName = null;
	private String originalFileName = null;

	public IdentificationDataFile(File xtandemFile) throws MSMSException {
		this.file = xtandemFile;
		try {
			String onlyFileName = this.file.getName();
			if (this.file.exists()) {
				// file path is OK
				originalFileName = null;
			} else {
				logger.debug("onlyFileName " + onlyFileName);
				String[] splitBackslash = onlyFileName.split("\\\\");
				if (splitBackslash.length > 1) {
					// the filename contains backslashes :
					// the path is encoded in Windows format
					String path = "";
					for (int i = 0; i < splitBackslash.length; i++) {
						path = path + File.pathSeparator + splitBackslash[i];
					}
					this.file = new File(path);
					onlyFileName = this.file.getName();
				}
			}
			fileName = onlyFileName;
		} catch (Exception e) {
			String message = "error instanciating IdentificationDataFile file : "
					+ this.file.getName() + " name ";
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	@Override
	public String getName() throws MSMSException {
		return fileName;
	}

	public String getAbsolutePath() {
		return this.file.getAbsolutePath();
	}

	@Override
	public boolean exists() {
		return this.file.exists();
	}

	@Override
	public int compareTo(IdentificationDataSource toCompare) {
		// logger.debug("compareTo");
		IdentificationDataFile dataFile = (IdentificationDataFile) toCompare;
		return this.file.getAbsolutePath().compareTo(
				dataFile.file.getAbsolutePath());
	}

	@Override
	public boolean equals(Object toCompare) {
		// logger.debug("equals");
		IdentificationDataFile dataFile = (IdentificationDataFile) toCompare;
		return this.file.getAbsolutePath().equals(
				dataFile.file.getAbsolutePath());
	}

	@Override
	public int hashCode() {
		// logger.debug("hashCode " + file.getAbsolutePath() + " "
		// + file.getAbsolutePath().hashCode());
		return file.getAbsolutePath().hashCode();
	}

	@Override
	abstract public Identification loadIdentification(
			Identification previousIdentification) throws Exception;

	@Override
	abstract public Identification loadPhosphoIdentification(
			Identification previousIdentification) throws Exception;

	@Override
	abstract public boolean loadMSMSSpectra(Peptide pep) throws Exception;

	@Override
	public void writeXpipPeptideSourceAttribute(XMLStreamWriter writer)
			throws Exception {
		if (originalFileName == null) {
			writer.writeAttribute("sample_file", this.file.getAbsolutePath());
		} else {
			writer.writeAttribute("sample_file", originalFileName);
		}
	}

}
