/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import java.io.File;

import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFile;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MzxmlFile;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerPepXmlResultFile;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerPepXmlResultPhosphoFile;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;

public class IdentificationPepXmlFile extends IdentificationDataFile {
	private static final Logger logger = Logger
			.getLogger(IdentificationPepXmlFile.class);
	private MzxmlFile mzXmlFile = null;
	private FastaFile fastaFile = null;

	public IdentificationPepXmlFile(File xtandemFile) throws MSMSException {
		super(xtandemFile);
	}

	@Override
	public Identification loadIdentification(Identification identification)
			throws MSMSException {
		logger.debug("IdentificationPepXmlFile loadIdentification begin");
		HandlerPepXmlResultFile pepxml = new HandlerPepXmlResultFile(
				identification, this);
		new parse_file(pepxml, this.file);
		identification = pepxml.get_identifications();
		this.mzXmlFile = pepxml.getMzxmlFile();
		this.fastaFile = pepxml.getFastaFile();
		setProteinSequences(identification, fastaFile);
		return identification;
	}

	@Override
	public Identification loadPhosphoIdentification(
			Identification identification) throws MSMSException {
		logger.debug("IdentificationPepXmlFile loadPhosphoIdentification begin");
		HandlerPepXmlResultPhosphoFile pepxml = new HandlerPepXmlResultPhosphoFile(
				identification, this);
		new parse_file(pepxml, this.file);
		identification = pepxml.get_identifications();
		this.mzXmlFile = pepxml.getMzxmlFile();
		this.fastaFile = pepxml.getFastaFile();
		setProteinSequences(identification, fastaFile);
		return identification;
	}

	static private void setProteinSequences(Identification identification,
			FastaFile fastaFile) throws MSMSException {
		// Set<String> accList = identification.get_protein_liste().keySet();
		logger.debug("setProteinSequences begin");
		if (fastaFile.exists()) {
			fastaFile.setProteinSequencesAndPeptidePositions(fastaFile,
					identification);
		} else {
			String message = "Fasta file '" + fastaFile.getAbsolutePath()
					+ "' does not exists\n"
					+ "unable to retrieve protein sequences";
			logger.warn(message);
		}
		logger.debug("setProteinSequences end");
	}

	@Override
	public boolean loadMSMSSpectra(Peptide pep) throws MSMSException {

		if (this.mzXmlFile == null) {
			// read the pep.xml file to get the mzXML file name ?
		} else {
			this.mzXmlFile.loadMSMSSpectra(pep);
			return true;
		}
		return false;

	}

	public void writeProticDbMlDataProcessing(XMLStreamWriter writer)
			throws Exception {
		logger.debug("writeProticDbMlDataProcessing begin");

		String message = "ERROR writeProticDbMlDataProcessing is not implemented in class IdentificationPepXmlFile";
		logger.error(message);
		throw new MSMSException(message);

	}

	@Override
	public void writeOdsMsSampleInformations(OdsDocWriterInterface odsTable,
			boolean xtandem_info_extract, boolean mzxml_info_extract)
			throws MSMSException {
		logger.debug("writeOdsInformations begin");

		if (!this.file.exists()) {
			try {
				odsTable.writeLine();
				odsTable.writeCell("pep XML result file NOT FOUND");
				odsTable.writeCell(this.file.getAbsolutePath());
			} catch (Exception e) {
				String message = "ERROR writing ODS in writeOdsMsSampleInformations "
						+ e.getMessage();
				logger.error(message);
				throw new MSMSException(message);
			}
		} else {
		}
	}

	@Override
	public String getSampleName() throws MSMSException {
		return this.getName().replaceAll("\\.pep\\.xml$", "");
	}

	@Override
	public void setScanCorrespondance(int fixedScanNum,
			int scanNumAsWrittenInDataSource) {

	}

	@Override
	public Integer getScanCorrespondance(int scanNumber) {
		return null;
	}
}
