/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms.MapDb;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;

public class PeptideProphet extends Peptide {
	/**
	 *
	 */
	private static final long serialVersionUID = -919182318722543799L;
	private static final Logger logger = Logger.getLogger(PeptideProphet.class);

	private Float prophetProbability = null;
	private Float interProphetProbability = null;
	private String searchEngine;

	@Override
	public int compareTo(Peptide p) {
		logger.debug("peptide prophet compar");
		int i;
		try {
			if (p == null) {
				throw new MSMSException("p is null");
			}
			i = this.getHashSampleScan().compareTo(p.getHashSampleScan());
		} catch (MSMSException e) {
			String message = e.getMessage();
			logger.error(message);
			i = 0;
		}
		if (i != 0)
			return (i);
		if (p.getClass().equals(PeptideProphet.class)) {
			PeptideProphet pepProph = (PeptideProphet) p;
			if ((this.interProphetProbability != null)
					&& (pepProph.getInterProphetProbability() != null)) {
				float delta = pepProph.getInterProphetProbability()
						- interProphetProbability;
				// float delta = this.evalue - p.get_evalue();
				if (delta > 0) {
					return (1);
				} else if (delta < 0) {
					return (-1);
				} else {
					return (0);
				}
			}

			if ((this.prophetProbability != null)
					&& (pepProph.getProphetProbability() != null)) {
				float delta = pepProph.getProphetProbability()
						- prophetProbability;
				// float delta = this.evalue - p.get_evalue();
				if (delta > 0) {
					return (1);
				} else if (delta < 0) {
					return (-1);
				} else {
					return (0);
				}
			}
		}

		// at least
		float delta = this.evalue - p.get_evalue();
		if (delta > 0) {
			return (1);
		} else if (delta < 0) {
			return (-1);
		} else {
			return (0);
		}
	}

	public PeptideProphet(Peptide currentPeptide) {
		logger.debug("new PeptideProphet");
		this.copy(currentPeptide);
	}

	public PeptideProphet() {
	}

	private void copy(Peptide currentPeptide) {
		sample_file = currentPeptide.sample_file;
		hashSampleScan = currentPeptide.hashSampleScan;
		RT = currentPeptide.RT;
		mhplus_obser = currentPeptide.mhplus_obser;
		mhplus_theo = currentPeptide.mhplus_theo;
		deltamass = currentPeptide.deltamass;
		sequence = currentPeptide.sequence;

		pre = currentPeptide.pre;
		post = currentPeptide.post;
		start = currentPeptide.start;
		stop = currentPeptide.stop;

		charge = currentPeptide.charge;
		evalue = currentPeptide.evalue;
		hypercorr = currentPeptide.hypercorr;
		modifList = currentPeptide.modifList;

		isValid = currentPeptide.isValid;
		mz_values = currentPeptide.mz_values;
		intensity_values = currentPeptide.intensity_values;
	}

	public void setProphetProbability(Double currentPeptideProphetProbability) {
		this.prophetProbability = currentPeptideProphetProbability.floatValue();
	}

	public void setInterProphetProbability(
			Double currentPeptideInterProphetProbability) {
		this.interProphetProbability = currentPeptideInterProphetProbability
				.floatValue();
	}

	public Float getProphetProbability() {
		return prophetProbability;
	}

	public Float getInterProphetProbability() {
		return interProphetProbability;
	}

	public void setSearchEngine(String currentSearchEngine) {
		this.searchEngine = currentSearchEngine;
	}

	public String getSearchEngine() {
		return searchEngine;
	}

	@Override
	protected void xpipWriteSpecificInfos(XMLStreamWriter writer)
			throws MSMSException {
		try {
			writer.writeStartElement("prophet");
			writer.writeAttribute("search_engine", this.getSearchEngine());
			if (this.getInterProphetProbability() != null) {
				writer.writeAttribute("inter_prophet_prob",
						"" + this.getInterProphetProbability());
			}
			if (this.getProphetProbability() != null) {
				writer.writeAttribute("prophet_prob",
						"" + this.getProphetProbability());
			}

			writer.writeEndElement();
		} catch (XMLStreamException e) {
			String message = "XMLStreamException ERROR writing XML in xpipWriteSpecificInfos "
					+ this.getClass() + "\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		} catch (Exception e) {
			String message = "Exception ERROR in xpipWriteSpecificInfos "
					+ this.getClass() + "\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		}

	}
}