package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.MSMSNotFoundException;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;

public class Modifs implements Comparable<Modifs>, Serializable,
		GrpPhosphoModif {
	private static final Logger logger = Logger.getLogger(Modifs.class);

	/**
	 *
	 */
	private static final long serialVersionUID = 676496558355723927L;
	private String AA;
	private int posi;
	private int posi_in_prot;
	private float modvalue;
	private boolean isotopic;

	/**
	 * Private method, same signature as in Serializable interface
	 *
	 * @param stream
	 * @throws IOException
	 */
	private void writeObject(ObjectOutputStream stream) throws IOException {
		logger.debug("writeObject begin");
		stream.writeObject("coucou");
	}

	private void readObject(ObjectInputStream stream) throws IOException,
			ClassNotFoundException {
		AA = (String) stream.readObject();
	}

	// constructeur
	public Modifs(float _modvalue) {
		this.modvalue = _modvalue;
		this.isotopic = false;
	}

	public Modifs(String _AA, int _posi, float _modvalue) {
		this.AA = _AA;
		this.posi = _posi;
		this.modvalue = _modvalue;
		this.isotopic = false;
		this.posi_in_prot = 0;
	}

	public int compareTo(Modifs p) {
		int k = (int) ((this.modvalue - p.get_modvalue()) * 100);
		return (k);
	}

	public String get_AA() {
		return (this.AA);
	}

	public int get_posi() {
		return (this.posi);
	}

	public float get_modvalue() {
		return (this.modvalue);
	}

	public String get_formater_value() {
		String f = this.AA + this.posi;
		if (this.modvalue > 0)
			f = f + ":+" + this.modvalue;
		else
			f = f + ":" + this.modvalue;
		return (f);
	}

	public boolean is_phopho_modifs() {
		if ((this.modvalue > 79.9) & (this.modvalue < 80.1))
			return true;
		boolean serthre = false;
		if (this.AA.equals("T"))
			serthre = true;
		else if (this.AA.equals("S"))
			serthre = true;
		if ((serthre) & (this.modvalue < -17.9) & (this.modvalue > -18.1))
			return true;
		return false;
	}

	public void setIsotopicLabel(boolean isotopic) {
		this.isotopic = isotopic;
	}

	public boolean isIsotopicLabel() {
		return isotopic;
	}

	public String toString() {
		if (modvalue == 0)
			return ("No isotope labelling");
		return ("" + modvalue);
	}

	public void setPosi_in_prot(int posi_in_prot) {
		this.posi_in_prot = posi_in_prot;
	}

	@Override
	public int getPosi_in_prot() {
		return posi_in_prot;
	}

	public boolean isNter() {
		if (this.posi == 1) {
			return true;
		}
		return false;
	}

	public PsiMod getPsiMod() throws MSMSNotFoundException {
		if (((get_modvalue() - 15.99) > -0.01)
				&& ((get_modvalue() - 15.99) < 0.01)) {
			return (PsiMod.getInstance("MOD:00719"));
			// oxidation to L-methionine sulfoxide [MOD:00719] 15.994915
		}
		if (((get_modvalue() - 57.02) > -0.03)
				&& ((get_modvalue() - 57.02) < 0.03)) {
			return (PsiMod.getInstance("MOD:00397"));
			// iodoacetamide [MOD:00397] 57.021464
		}
		if (((get_modvalue() - 42.01) > -0.01)
				&& ((get_modvalue() - 42.01) < 0.01)) {
			return (PsiMod.getInstance("MOD:00408"));
			// N-acetylated residue [MOD:00408] 42.010565
		}
		if (((get_modvalue() - 79.966) > -0.01)
				&& ((get_modvalue() - 79.966) < 0.01)) {
			return (PsiMod.getInstance("MOD:00696"));
			// phosphorylated residue [MOD:00696] 79.966330
		}
		if (((get_modvalue() + 17.026) > -0.01)
				&& ((get_modvalue() + 17.026) < 0.01)) {
			return (PsiMod.getInstance("MOD:01160"));
			// loss of ammonia [MOD:01160] -17.026549
		}
		if (((get_modvalue() + 18.01) > -0.01)
				&& ((get_modvalue() + 18.01) < 0.01)) {
			return (PsiMod.getInstance("MOD:00704"));
			// dehydrated residue [MOD:00704] -18.010565
		}

		// if (mod.isNter()) {
		// dimethylation
		if (((get_modvalue() - 28.031300) > -0.01)
				&& ((get_modvalue() - 28.031300) < 0.01)) {
			return (PsiMod.getInstance("MOD:00429"));
			// id: MOD:00429 name: dimethylated residue xref: DiffMono:
			// "28.031300" xref: DiffFormula: "C 2 H 4"
		}
		// inter dimethylation
		if (((get_modvalue() - 32.056407) > -0.01)
				&& ((get_modvalue() - 32.056407) < 0.01)) {
			return (PsiMod.getInstance("MOD:00552"));
			// id: MOD:00552 name: 4x(2)H labeled dimethyl residue xref:
			// DiffMono: "32.056407" xref: DiffFormula: "C 2 (2)H 4"
		}
		// }
		// heavy dimethylation

		// if (mod.get_AA() == "R") {
		if (((get_modvalue() - 36.075670) > -0.01)
				&& ((get_modvalue() - 36.075670) < 0.01)) {

			return (PsiMod.getInstance("MOD:00638"));
			// id: MOD:00638 name: 2x(13)C,6x(2)H-dimethylated arginine
			// xref:
			// DiffMono: "36.075670" xref: DiffFormula:
			// "(13)C 2 (1)H -2 (2)H 6"
		}

		if (((get_modvalue() - 324.105647 ) > -0.01)
				&& ((get_modvalue() - 324.105647 ) < 0.01)) {
			return (PsiMod.getInstance("MOD:00628"));
			// Hex2    DimethylpyrroleAdduct  [MOD:00628] 324.105647
		}

		String message = "modif " + this.get_modvalue()
				+ " not known in PsiMod";
		logger.error(message);
		throw new MSMSNotFoundException(message);
	}
}