/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>,
 * Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.output.XpipOutput;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXpipFile;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;
import fr.inra.pappso.xtandempipeline.thread.XtpProcess;

public class XpipFile {
	private static final Logger logger = Logger.getLogger(XpipFile.class);

	private File xpipFile = null;
	private URL urlOfXpipData = null;

	private boolean modified = false;

	public XpipFile(String fileName) {
		xpipFile = new File(fileName);
	}

	public XpipFile(URL urlOfXpipData) {
		logger.debug("urlOfXpipData " + urlOfXpipData.toString());
		if (urlOfXpipData.getProtocol().equals("file")) {
			File test = new File(urlOfXpipData.getFile());
			if (test.exists()) {
				xpipFile = test;
				urlOfXpipData = null;
				logger.debug("xpipFile " + xpipFile.getAbsolutePath());
				return;
			}
		}
		xpipFile = null;
		this.urlOfXpipData = urlOfXpipData;

	}

	public String getName() {
		if (xpipFile == null) {
			return urlOfXpipData.getFile();
		}
		return xpipFile.getName();
	}

	public boolean exists() {
		if (xpipFile == null) {
			return false;
		}
		return xpipFile.exists();
	}

	public String getAbsolutePath() {
		if (xpipFile == null) {
			return urlOfXpipData.toString();
		}
		return xpipFile.getAbsolutePath();
	}

	public void write(XtpProcess save_result_and_filtering,
			XtandemPipelineSession session) throws Exception {
		if (xpipFile == null) {
			String message = "Unable to write XPIP data to an URL ("
					+ urlOfXpipData.toString()
					+ "). Try \"Save project as\" instead.";
			logger.debug(message);
			throw new MSMSException(message);
		}
		XpipOutput xpipOutput = new XpipOutput();
		OutputStream outputStream = new FileOutputStream(this.xpipFile);
		xpipOutput.open(outputStream);

		xpipOutput.write(session.getIdentificationList(),
				session.getDataTypeBase(), save_result_and_filtering);

		xpipOutput.close();
		/*
		 * save_project save = new save_project(session.getIdentificationList(),
		 * session.getDataTypeBase(), save_result_and_filtering); if (xpipFile
		 * == null) { String message = "Unable to write XPIP data to an URL (" +
		 * urlOfXpipData.toString() + "). Try \"Save project as\" instead.";
		 * logger.debug(message); throw new MSMSException(message); }
		 * save.write_data_to(xpipFile);
		 */
		this.modified = false;
	}

	private InputStream getInputStream() throws MSMSException {
		if (xpipFile == null) {
			URLConnection uc;
			try {
				uc = urlOfXpipData.openConnection();
			} catch (IOException e) {
				String message = "error opening XPIP url "
						+ urlOfXpipData.toString();
				logger.debug(message);
				throw new MSMSException(message);
			}

			XtandemPipelineSession.getInstance().setHttpAuthorization(uc);

			try {
				return uc.getInputStream();
			} catch (IOException e) {
				String message = "error getting XPIP url inputStream "
						+ urlOfXpipData.toString();
				logger.debug(message);
				throw new MSMSException(message);
			}
		} else {
			try {
				return new FileInputStream(this.xpipFile);
			} catch (FileNotFoundException e) {
				String message = "XPIP file " + this.xpipFile.getAbsolutePath()
						+ " not found";
				logger.debug(message);
				throw new MSMSException(message);
			}
		}
	}

	public void read(HandlerXpipFile load) throws MSMSException, Exception {
		new parse_file(load, this.getInputStream());
		this.modified = false;
	}

	public void modified() {
		// content midified but not saved
		this.modified = true;
	}

	public boolean isModified() {
		return this.modified;
	}

}
