package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFile;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.GroupSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class FilterGroupFromFastaDatabase {

	private static final Logger logger = Logger
			.getLogger(FilterGroupFromFastaDatabase.class);

	private FastaFile database;

	public FilterGroupFromFastaDatabase(FastaFile database) {
		logger.info("Remove Group from Fasta Database " + database.getName());
		this.database = database;
	}

	public void filter(GroupSet grouping) throws MSMSException {
		// Read fasta file to get accession list
		if (database.exists()) {
			logger.info("Read fasta file : " + database.getName());

			HashSet<String> access = database.getAccessionSet();
			// Search contaminants proteins
			logger.info("Remove Group in new mode");
			for (Group group : grouping.getGroupList()) {
				boolean isconta = false;
				for (SubGroup sg : group.getSubGroupSet()) {
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtSet()) {
						if (access.contains(hashProt.getGrpProtMatch()
								.getGrpProtein().getAccession())) {
							isconta = true;
						}
					}
				}
				if (isconta)
					grouping.getHashSetOfGroups().remove(group);
			}
		} else {
			logger.error("fasta file doesnt not exist: "
					+ database.getAbsolutePath());
			logger.info("Remove Group from Database in old mode");
			for (Group group : grouping.getGroupList()) {
				boolean isconta = false;
				for (SubGroup sg : group.getSubGroupSet()) {
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtSet()) {
						if (((Match) hashProt.getGrpProtMatch())
								.get_protein_match().getDatabase()
								.getDatabasePath().contains(database.getName())) {
							isconta = true;
						}
					}
				}
				if (isconta)
					grouping.getHashSetOfGroups().remove(group);
			}
		}

		// Numbering after contaminant removing
		grouping.numbering();
	}

}
