package fr.inra.pappso.xtandempipeline.grouping;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpMsSample;

public class HashSampleScanSet extends HashSet<HashSampleScan> implements
		Comparable<HashSampleScanSet> {
	/**
	 *
	 */
	private static final long serialVersionUID = -9055338154900110371L;
	private static final Logger logger = Logger
			.getLogger(HashSampleScanSet.class);

	// String to unique key for sample scan
	private HashMap<String, HashSampleScan> mapHashSampleScan = new HashMap<String, HashSampleScan>(
			0);

	public HashSampleScanSet() {
	}

	public int compareTo(HashSampleScanSet arg0) {
		// if i>0, arg0 contains more sample scan than this, tells that arg0 < this
		// if i<0, arg0 contains less sample scan than this, tells that arg0 > this
		// => this sorts bigger sets first, little sets at the end
		int i = arg0.size() - this.size();
		if (i != 0)
			return i;
		return (this.getFingerCode() - arg0.getFingerCode());
	}

	public HashSampleScan getInstance(int scan, GrpMsSample msRun) {
		String tempString = msRun.getSampleName() + "-" + scan;
		HashSampleScan temp = this.mapHashSampleScan.get(tempString);
		if (temp == null) {
			// this HashSampleScan is new
			temp = new HashSampleScan(scan, msRun);
			this.mapHashSampleScan.put(tempString, temp);
			boolean test = this.add(temp);
			if (!test) {
				logger.error("HashSampleScan COLLISION, must never appear");
			}
			return (temp);
		} else {
			// find the existing instance of the same sample scan :
			return temp;
		}

	}

	public void loggerDebug() {
		logger.debug("HashSampleScanSet :");
		for (HashSampleScan scan : this) {
			logger.debug(scan.toString() + " " + scan.hashCode());
		}
		// return null;
	}

	public ArrayList<HashSampleScan> getHashSampleScanList() {
		ArrayList<HashSampleScan> array = new ArrayList<HashSampleScan>(this);
		Collections.sort(array);
		return (array);
	}

	final private int getFingerCode() {
		int finger = 0;
		for (HashSampleScan sampleScan : this) {
			finger += sampleScan.getScan();
		}
		return (finger);
	}

	@Override
	public void clear() {
		mapHashSampleScan = new HashMap<String, HashSampleScan>(0);
		super.clear();
	}

}
