package fr.inra.pappso.xtandempipeline.grouping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPeptide;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpProtMatch;

public class HashSampleScanSetPhospho extends HashSampleScanSetProt {
	/**
	 * 
	 */
	private static final long serialVersionUID = 4256968597752695888L;

	/**
	 * 
	 */
	private static final Logger logger = Logger
			.getLogger(HashSampleScanSetPhospho.class);

	private TreeSet<Integer> phosphoPositions = new TreeSet<Integer>();

	public HashSampleScanSetPhospho(GrpProtMatch match,
			ArrayList<Integer> phosphoPositions) throws Exception {
		super();
		this.setGrpProtMatch(match);
		this.phosphoPositions.addAll(phosphoPositions);
		for (GrpPeptide peptide : match.getGrpPeptides()) {
			for (Integer position : phosphoPositions) {
				if (peptide.containsPosition(position)) {
					HashSampleScan hashSample = peptide.getHashSampleScan();
					if (hashSample == null) {
						String message = "hashSample is null";
						logger.error(message);
					}
					this.add(hashSample);
				}
			}
		}

	}

	@Override
	public void clear() {
		phosphoPositions = new TreeSet<Integer>();
		super.clear();
	}

	/*
	 * @Override public int compareTo(HashSampleScanSet o) { int i = o.size() -
	 * this.size(); if (i != 0) return i; return (o.hashCode() -
	 * this.hashCode()); }
	 */
	public TreeSet<Integer> getPhosphoPositions() {
		return (phosphoPositions);
	}

	public String getPhosphoPositionToPrint() {
		String S = "";
		for (Integer value : this.phosphoPositions) {
			S = S + " " + value;
		}
		S.replaceFirst(" ", "");
		return (S);
	}

	public boolean haveMultiPhosphoPeptide() throws MSMSException {
		boolean multi = false;
		for (HashSampleScan hash : this) {
			if (this.match.getBestPeptideToHashSampleScan(hash)
					.getGrpPhosphoModifs().size() > 1)
				multi = true;
		}
		return (multi);
	}

	@Override
	public HashKey4GroupsSet getHashKey4GroupsSet() throws Exception {
		HashKey4GroupsSet hashPhosphoProtSet = new HashKey4GroupsSet();

		HashProtein hashProtein = match.getHashProtein();

		hashPhosphoProtSet.add(hashProtein);

		return hashPhosphoProtSet;
	}

	public TreeSet<Integer> getAllPositionIncluderedon(HashSampleScan hash)
			throws MSMSException {
		TreeSet<Integer> posi = new TreeSet<Integer>();

		for (GrpPhosphoModif mods : getAllPhosphoPeptideModif(hash)) {
			posi.add(mods.getPosi_in_prot());
		}

		return posi;
	}

	/**
	 * retrieve all Phospho Modifs concerning one Peptide
	 * 
	 * @param peptide
	 * @return
	 * @throws MSMSException
	 */
	public Collection<GrpPhosphoModif> getAllPhosphoPeptideModif(Peptide peptide)
			throws MSMSException {
		return getAllPhosphoPeptideModif(peptide.getHashSampleScan());
	}

	private Collection<GrpPhosphoModif> getAllPhosphoPeptideModif(
			HashSampleScan hash) throws MSMSException {

		ArrayList<GrpPhosphoModif> modifs = new ArrayList<GrpPhosphoModif>();
		for (GrpPeptide pepTemp : this.match
				.getGrpPeptidesSharingHashSampleScan(hash)) {
			modifs.addAll(pepTemp.getGrpPhosphoModifs());
		}
		return modifs;
	}

	/**
	 * retrieve all Phospho Modifs concerning one PeptideMass
	 * 
	 * @param peptideMass
	 * @return
	 * @throws MSMSException
	 */

	public Collection<GrpPhosphoModif> getAllPhosphoModif(
			PeptideMass peptideMass) throws MSMSException {

		ArrayList<GrpPhosphoModif> modifs = new ArrayList<GrpPhosphoModif>();
		for (HashSampleScan sampleScan : peptideMass.getHashSampleScanSet()) {
			modifs.addAll(getAllPhosphoPeptideModif(sampleScan));
		}
		return modifs;
	}

}
