/*******************************************************************************
 * Copyright (c) 2012 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;
import fr.inra.moulon.proticport.model.exceptions.ProticPortException;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.XtandemPipelineMain;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.PsiMod;
import fr.inra.pappso.xtandempipeline.class_msms.XpipFile;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationXtandemFile;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class ProticDbMlWriter {
	private static final Logger logger = Logger
			.getLogger(ProticDbMlWriter.class);

	static String namespaceURI = "http://sashimi.sourceforge.net/schema_revision/mzXML_2.0";
	static String xmlnsxsi = "http://www.w3.org/2001/XMLSchema-instance";
	static String xsischemaLocation = "http://uln.moulon.inra.fr/~langella/proticdev/doc/xml_schema/proticdbml/proticdbml.xsd";

	protected XMLStreamWriter writer;

	private ArrayList<Identification> arrayIdentifications = new ArrayList<Identification>();

	private Hashtable<String, String> prot_to_id = new Hashtable<String, String>();

	private Hashtable<String, String> pepSampleScanCle2xmlId = new Hashtable<String, String>();

	private Hashtable<String, String> sample_to_id = new Hashtable<String, String>();

	private Hashtable<String, String> msrun_to_id = new Hashtable<String, String>();

	private long duracel;

	private XpipFile xpipFile;

	private String xtpExperimentType;

	ProticDbMlWriter(XMLStreamWriter writer) throws ProticPortException {
		this.writer = writer;

		try {
			this.initialize();
		} catch (XMLStreamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	protected void initialize() throws XMLStreamException, ProticPortException {
		logger.debug("write begin");
		duracel = System.currentTimeMillis();
		// xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		writer.setPrefix("xsi", xmlnsxsi);
		// writer.setDefaultNamespace(namespaceURI);
		writer.writeStartElement("PROTICdb");
		writer.writeNamespace("xsi", xmlnsxsi);
		// writer.writeDefaultNamespace(namespaceURI);

		// writer.writeAttribute(xmlnsxsi, "schemaLocation", xsischemaLocation);
		// writer.writeAttribute("xmlns", "xsi", xmlnsxsi);

		writer.writeAttribute(xmlnsxsi, "noNamespaceSchemaLocation",
				xsischemaLocation);

		writer.writeAttribute("version", "1.0");

		writer.writeAttribute("type", "MSidentificationResults");

		writeIdentMethod();
		writer.writeStartElement("sequences");

	}

	void write(Identification identification) {
		// protein_list
		logger.debug("write begin");
		try {

			this.arrayIdentifications.add(identification);

			// this.sequences = doc.createElement("sequences");
			this.writeSequence(identification);

		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");
	}

	public void close() throws Exception {
		writer.writeEndElement();// sequences

		writer.writeStartElement("project");
		writer.writeAttribute("name", "");
		writer.writeAttribute("id", "p1");
		// ajout des echantillons_msrun
		this.writeSamples();
		this.writeMsRuns();

		writer.writeStartElement("identificationRuns");

		for (Identification identification : this.arrayIdentifications) {
			writer.writeStartElement("identificationRun");
			// ajout d'une mnip d'identification
			this.writeIdentificationRun(identification);

			// ajout des petides
			this.writepeptideHits(identification);

			// ajout des mathces
			this.writeMatchs(identification);

			writer.writeEndElement();// identificationRun
		}
		writer.writeEndElement();// identificationRuns
		writer.writeEndElement();// project

		writer.writeEndDocument();

		writer.flush();
		writer.close();

		logger.info("Duration creating PROTICdbML xml document: "
				+ (System.currentTimeMillis() - duracel) + " ms");
	}

	private void writeIdentMethod() {
		logger.debug("write begin");
		try {
			// Ajout des méthodes
			writer.writeStartElement("identMeth");
			writer.writeAttribute("name", "");
			writer.writeAttribute("id", "m1");
			writer.writeEndElement();

			// Ajout des base de données
			writer.writeStartElement("customDb");
			writer.writeAttribute("name", "");
			writer.writeAttribute("id", "customdb0");
			writer.writeEndElement();

		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");

	}

	private void writeSamples() throws Exception {
		try {
			writer.writeStartElement("samples");

			for (Identification ident : arrayIdentifications) {
				for (MsRun samp : ident.getMsRunSet()) {
					writer.writeStartElement("sample");
					String id_samp = "samp" + sample_to_id.size();
					String id_msrun = "msr" + msrun_to_id.size();
					String name = samp.getSampleName();
					name = name.replaceAll(".xml", "");
					// balise sample
					writer.writeAttribute("name", name);
					writer.writeAttribute("id", id_samp);
					writer.writeEmptyElement("description");
					sample_to_id.put(samp.getSampleName(), id_samp);

					// Element spectrumList = doc.createElement("spectrumList");
					// msRun.appendChild(spectrumList);

					msrun_to_id.put(samp.getSampleName(), id_msrun);
					writer.writeEndElement();// sample
				}
			}
			writer.writeEndElement();// samples
		} catch (Exception e) {
			String message = "error writing samples in ProticDbMl \n"
					+ e.getMessage();
			logger.error(message);
			throw new Exception(message);
		}
	}

	private void writeMsRuns() throws Exception {
		try {
			writer.writeStartElement("msRuns");

			for (Identification ident : arrayIdentifications) {
				for (MsRun samp : ident.getMsRunSet()) {

					String name = samp.getSampleName();
					name = name.replaceAll(".xml", "");

					String id_samp = sample_to_id.get(samp.getSampleName());

					// Element spectrumList = doc.createElement("spectrumList");
					// msRun.appendChild(spectrumList);

					String id_msrun = msrun_to_id.get(samp.getSampleName());

					// balise MsRun
					writer.writeStartElement("msRun");
					writer.writeAttribute("sample_id", id_samp);
					writer.writeAttribute("id", id_msrun);

					writer.writeStartElement("description");
					writer.writeStartElement("admin");
					// writer.writeStartElement("contact");
					// writer.writeStartElement("email");
					// writer.writeCharacters("valot@moulon.inra.fr");
					// writer.writeEndElement();// email

					// writer.writeStartElement("name");
					// writer.writeCharacters("Valot Benoit");
					// writer.writeEndElement();// name
					// writer.writeEndElement();// contact

					writer.writeStartElement("sourceFile");
					writer.writeStartElement("nameOfFile");
					writer.writeCharacters(name + ".RAW");
					writer.writeEndElement();// nameOfFile
					writer.writeEndElement();// sourceFile

					writer.writeStartElement("sampleName");
					writer.writeCharacters(name);
					writer.writeEndElement();// sampleName
					writer.writeEndElement();// admin
					writer.writeEndElement();// description

					writer.writeEndElement();// msRun
				}
			}
			writer.writeEndElement();// msRuns
		} catch (Exception e) {
			String message = "error writing MS runs in ProticDbMl \n"
					+ e.getMessage();
			logger.error(message);
			throw new Exception(message);
		}
	}

	private void writeSequence(Identification ident) throws Exception {
		try {
			for (Group group : ident.getGrouping().getGroupList()) {
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						Protein prot = (Protein) hashProt.getGrpProtMatch()
								.getGrpProtein();
						if (prot_to_id.containsKey(prot.get_description()) == false) {
							String id = "seq" + prot_to_id.size();
							writer.writeStartElement("sequence");
							writer.writeAttribute("id", id);
							if (prot.get_description().length() > 60) {
								writer.writeAttribute("display_id", prot
										.get_description().substring(0, 59));
							} else {
								writer.writeAttribute(
										"display_id",
										prot.get_description()
												.substring(
														0,
														prot.get_description()
																.length()));
							}
							// <dbxref key="AT5G16390"
							// dbname="AGI_LocusCode"></dbxref>
							if (prot.get_dbxref_type().equals("no") == false) {
								writer.writeStartElement("dbxref");
								writer.writeAttribute("key",
										prot.get_dbxref_accession());
								writer.writeAttribute("dbname",
										prot.get_dbxref_type());
								writer.writeEndElement();// dbxref
							}
							writer.writeStartElement("description");
							writer.writeCharacters(prot.get_description());
							writer.writeEndElement();// description
							writer.writeStartElement("seq");
							writer.writeCharacters(prot.get_sequence());
							writer.writeEndElement();// seq

							writer.writeEndElement();// sequence

							prot_to_id.put(prot.get_description(), id);
						}
					}
				}
			}

		} catch (Exception e) {
			String message = "error writing sequences in ProticDbMl \n"
					+ e.getMessage();
			logger.error(message);
			throw new Exception(message);
		}
	}

	private void writeIdentificationRun(Identification identification)
			throws Exception {
		try {
			writer.writeAttribute("ident_method_id", "m1");
			writer.writeAttribute("customdb_id", "customdb0");

			writer.writeStartElement("description");
			writer.writeStartElement("admin");
			// writer.writeStartElement("contact");
			// writer.writeStartElement("email");
			// writer.writeCharacters("valot@moulon.inra.fr");
			// writer.writeEndElement();// email

			// writer.writeStartElement("name");
			// writer.writeCharacters("Valot Benoit");
			// writer.writeEndElement();// name
			// writer.writeEndElement();// contact
			this.xpipFile = XtandemPipelineSession.getInstance()
					.getCurrentXpipFile();

			writer.writeStartElement("sourceFile");
			writer.writeStartElement("nameOfFile");
			// TODO
			// get the loaded xpip file name
			writer.writeCharacters(this.xpipFile.getName());
			writer.writeEndElement();// nameOfFile
			writer.writeStartElement("pathToFile");
			writer.writeCharacters(this.xpipFile.getAbsolutePath());
			writer.writeEndElement();// pathToFile
			writer.writeStartElement("fileType");
			writer.writeCharacters("XPIP file");
			writer.writeEndElement();// fileType
			writer.writeEndElement();// sourceFile

			writer.writeEndElement();// admin

			// if we can retrieve original informations in xml xtandem results
			HashSet<IdentificationXtandemFile> xtSourceFiles = Utils
					.getXtandemFileList(identification);
			if (xtSourceFiles != null) {
				//
				for (IdentificationXtandemFile xtFile : xtSourceFiles) {
					xtFile.writeProticDbMlDataProcessing(this.writer);
				}
			} else {
				writer.writeStartElement("dataProcessing");
				writer.writeStartElement("software");
				writer.writeStartElement("name");
				writer.writeAttribute("acc", "PROTICdbO:0000283");
				writer.writeCharacters("X!Tandem");
				writer.writeEndElement();// name
				writer.writeStartElement("version");
				writer.writeCharacters(XtandemPipelineSession.getInstance()
						.getConfig().getXtandemVersion());
				writer.writeEndElement();// version
				writer.writeEndElement();// software
				writer.writeStartElement("processingMethod");
				writer.writeEndElement();// processingMethod
				writer.writeEndElement();// dataProcessing
			}

			// id: PROTICdbO:0000316
			// name: X!TandemPipeline

			writer.writeStartElement("dataProcessing");
			writer.writeStartElement("software");
			writer.writeStartElement("name");
			writer.writeAttribute("acc", "PROTICdbO:0000316");
			writer.writeCharacters("X!TandemPipeline");
			writer.writeEndElement();// name
			writer.writeStartElement("version");
			writer.writeCharacters(XtandemPipelineMain.version);
			writer.writeEndElement();// version
			writer.writeEndElement();// software
			writer.writeStartElement("processingMethod");
			// cvParams

			this.writeCvParam("PROTICdbO:0000323", ""
					+ XtandemPipelineSession.getInstance().getConfig()
							.get_protein_evalue(),
					"X!TandemPipeline filter on protein evalue (log)");

			// indi
			// combine
			// phospho
			xtpExperimentType = XtandemPipelineSession.getInstance()
					.getDataTypeBase().getType();
			if (xtpExperimentType.equals("indiv")) {
				this.writeCvParam("PROTICdbO:0000319", xtpExperimentType, "");
			}
			if (xtpExperimentType.equals("combi")) {
				this.writeCvParam("PROTICdbO:0000320", xtpExperimentType, "");
			}
			if (xtpExperimentType.equals("phospho")) {
				this.writeCvParam("PROTICdbO:0000321", xtpExperimentType, "");
			}

			this.writeCvParam("PROTICdbO:0000325", ""
					+ XtandemPipelineSession.getInstance().getConfig()
							.get_peptide_number(),
					"X!TandemPipeline filter on minimal number of peptide per protein");

			this.writeCvParam("PROTICdbO:0000324", ""
					+ XtandemPipelineSession.getInstance().getConfig()
							.get_peptide_evalue(),
					"X!TandemPipeline filter on peptide evalue");

			// TODO write database filter
			// this.writeCvParam("PROTICdbO:0000324", ""
			// +
			// XtandemPipelineSession.getInstance().getConfig().get_database_filter(),
			// "X!TandemPipeline filter on peptide evalue");

			writer.writeEndElement();// processingMethod

			writer.writeEndElement();// dataProcessing

			writer.writeEndElement();// description
		} catch (Exception e) {
			String message = "error writing identificationRun in ProticDbMl \n"
					+ e.getMessage();
			logger.error(message);
			throw new Exception(message);
		}
	}

	private void writePeptideHit(Peptide pep, Group group,
			PeptideMass peptideMass) throws XMLStreamException, MSMSException {

		if (peptideMass == null) {
			String message = "peptideMass == null";
			logger.error(message);
			throw new MSMSException(message);
		}

		String pep_hit_id = "pep" + pepSampleScanCle2xmlId.size();
		// param par default
		writer.writeStartElement("peptideHit");
		writer.writeAttribute("calc_mr", "" + (pep.get_mhplus_theo() - 1.00794));
		double exp_mz = (pep.get_mhplus_obser() + (1.00794 * (pep.getCharge() - 1)))
				/ pep.getCharge();
		writer.writeAttribute("exp_mz", "" + exp_mz);
		writer.writeAttribute("delta", "" + pep.get_deltamass());
		writer.writeAttribute("exp_mr", "" + (pep.get_mhplus_obser() - 1.00794));
		writer.writeAttribute("acq_number", "" + pep.get_scan());
		writer.writeAttribute("ms_run_id",
				"" + msrun_to_id.get(pep.get_sample()));
		writer.writeAttribute("id", pep_hit_id);
		writer.writeAttribute("exp_z", "" + pep.getCharge());

		this.writeCvParam("PROTICdbO:0000339",
				Utils.getPappsoPeptideMassId(group, peptideMass),
				"peptide mass id");

		if (xtpExperimentType.equals("phospho")) {
			this.writeCvParam("PROTICdbO:0000349",
					Utils.getPappsoPhosphoPeptideMassId(group, peptideMass),
					"phosphopeptide mass id");
		}

		// pep.getMsRun().
		IdentificationDataSource idDataSource = pep
				.getIdentificationDataSource();
		if (idDataSource.getClass().equals(IdentificationXtandemFile.class)) {
			// cvparam specifique xtandem

			this.writeCvParam("PROTICdbO:0000287", "" + pep.get_evalue(),
					"xtandem peptide evalue");

			this.writeCvParam("PROTICdbO:0000288", "" + pep.get_hypercorr(),
					"xtandem peptide hyperscore");
		}

		this.writeCvParam("PROTICdbO:0000289",
				"" + pep.getRetentionTimeInMinutes(),
				"xtandem peptide retention time");
		/*
		 * writer.writeStartElement("cvParam"); writer.writeAttribute("value",
		 * "" + pep.get_RT()); writer.writeAttribute("name",
		 * "xtandem peptide retention time"); writer.writeAttribute("cvLabel",
		 * "PROTICdbO"); writer.writeAttribute("accession",
		 * "PROTICdbO:0000289"); writer.writeEndElement();// cvParam
		 */

		// Element cvParam4 = doc.createElement("cvParam");
		// cvParam4.setAttribute("value", "" + pep.get_post());
		// cvParam4.setAttribute("name", "xtandem peptide post");
		// cvParam4.setAttribute("cvLabel", "PROTICdbO");
		// cvParam4.setAttribute("accession", "PROTICdbO:00000XX");
		// peptideHit.appendChild(cvParam4);
		//
		// Element cvParam5 = doc.createElement("cvParam");
		// cvParam5.setAttribute("value", "" + pep.get_pre());
		// cvParam5.setAttribute("name", "xtandem peptide pre");
		// cvParam5.setAttribute("cvLabel", "PROTICdbO");
		// cvParam5.setAttribute("accession", "PROTICdbO:00000XX");
		// peptideHit.appendChild(cvParam5);

		// sequences avec les modifs
		writer.writeStartElement("pepSeq");
		writer.writeStartElement("peptide");
		writer.writeCharacters("" + pep.getSequence());
		writer.writeEndElement();// peptide
		if (pep.get_Modifs().size() != 0) {
			this.writePtms(pep.get_Modifs());
		}
		writer.writeEndElement();// pepSeq

		pepSampleScanCle2xmlId.put(pep.get_sample_scan_sequencepeptide_cle(),
				pep_hit_id);
		writer.writeEndElement();// peptideHit
	}

	private void writePtms(ArrayList<Modifs> mods) throws XMLStreamException {
		writer.writeStartElement("ptms");
		for (Modifs mod : mods) {
			writer.writeStartElement("ptm");
			writer.writeAttribute("diff_mono", "" + mod.get_modvalue());
			writer.writeAttribute("position", "" + mod.get_posi());
			writer.writeAttribute("aa", "" + mod.get_AA());
			try {
				this.writeOboModif(mod.getPsiMod());
			} catch (MSMSException e) {
				String message = "error writing OBO modif : " + e.getMessage();
				logger.warn(message);
			}
			writer.writeEndElement();// ptm

		}
		writer.writeEndElement();// ptms
	}

	private void writeOboModif(PsiMod mod) throws XMLStreamException {
		writer.writeStartElement("cvParam");
		writer.writeAttribute("name", mod.getName());
		writer.writeAttribute("cvLabel", mod.getCvLabel());
		writer.writeAttribute("accession", mod.getAccession());
		writer.writeEndElement();// cvParam
	}

	private void writepeptideHits(Identification identification)
			throws MSMSException {
		try {
			writer.writeStartElement("peptideHits");
			for (Group group : identification.getGrouping().getGroupList()) {
				// generating peptide numbers
				PeptideMassSet pepMassSet = new PeptideMassSet();

				Hashtable<String, PeptideMass> pepSampleScanCle2peptideMass;
				if (this.xtpExperimentType.equals("phospho")) {
					pepSampleScanCle2peptideMass = pepMassSet
							.numberingPhosphoGetSampleScanCle2peptideMass(group);

				} else {
					pepSampleScanCle2peptideMass = pepMassSet
							.numberingGetSampleScanCle2peptideMass(group);
				}

				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {

						ArrayList<Peptide> peptideList = null;
						if (this.xtpExperimentType.equals("phospho")) {
							peptideList = ((Match) hashProt.getGrpProtMatch())
									.getValidPeptideInPhosphoIsland((HashSampleScanSetPhospho) hashProt);
						} else {
							peptideList = ((Match) hashProt.getGrpProtMatch())
									.get_valid_peptides();
						}

						for (Peptide pep : peptideList) {

							if (pepSampleScanCle2xmlId.containsKey(pep
									.get_sample_scan_sequencepeptide_cle()) == false) {

								writePeptideHit(
										pep,
										group,
										pepSampleScanCle2peptideMass.get(pep
												.get_sample_scan_sequencepeptide_cle()));

							}
						}
					}
				}
			}
			writer.writeEndElement();// "peptideHits");
		} catch (Exception e) {
			String message = "error writing peptideHits in ProticDbMl \n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeMatchs(Identification identification)
			throws MSMSException {
		try {
			writer.writeStartElement("matchs");
			for (Group group : identification.getGrouping().getGroupList()) {
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					writer.writeStartElement("match");

					// echantillons
					Hashtable<String, String> view = new Hashtable<String, String>();
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						Match match = (Match) hashProt.getGrpProtMatch();
						for (MsRun sample : match.get_sample_matchs()) {
							if (!view.containsKey(sample.getSampleName())) {
								writer.writeStartElement("matchSample");
								writer.writeAttribute(
										"sample_id",
										""
												+ sample_to_id.get(sample
														.getSampleName()));
								String name = sample.getSampleName();
								name = name.replaceAll(".xml", "");
								writer.writeAttribute("name", "" + name);
								view.put(sample.getSampleName(), "ok");
								writer.writeEndElement();// "matchSample");
							}
						}
					}

					// proteins
					int k = 1;
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						writer.writeStartElement("proteinHit");
						Protein prot = (Protein) hashProt.getGrpProtMatch()
								.getGrpProtein();
						writer.writeAttribute("sequence_id",
								"" + prot_to_id.get(prot.get_description()));
						// proteinHit.setAttribute("score", "");
						writer.writeAttribute("rank", "" + k);

						// //cvparam
						this.writeCvParam("PROTICdbO:0000284",
								"" + prot.get_mw(),
								"MW determination by Benuch");
						/*
						 * writer.writeStartElement("cvParam");
						 * writer.writeAttribute("value", "" + prot.get_mw());
						 * writer.writeAttribute("name",
						 * "MW determination by Benuch");
						 * writer.writeAttribute("cvLabel", "PROTICdbO");
						 * writer.writeAttribute("accession",
						 * "PROTICdbO:0000284"); writer.writeEndElement();//
						 * cvParam
						 */
						// evalue
						MsRunSet samples = ((Match) hashProt.getGrpProtMatch())
								.get_sample_matchs();
						this.writeCvParam("PROTICdbO:0000291",
								"" + prot.get_evalue(), "Xtandem log evalue");
						/*
						 * writer.writeStartElement("cvParam");
						 * writer.writeAttribute("value", "" +
						 * prot.get_evalue());
						 *
						 * writer.writeAttribute("name", "Xtandem log evalue");
						 * writer.writeAttribute("cvLabel", "PROTICdbO");
						 * writer.writeAttribute("accession",
						 * "PROTICdbO:0000291"); writer.writeEndElement();//
						 * cvParam
						 */

						// coverage
						if (samples.size() == 1) {
							this.writeCvParam(
									"PROTICdbO:0000285",
									""
											+ ((Match) hashProt
													.getGrpProtMatch())
													.get_coverage(),
									"protein coverage");
						}

						// [Term]
						// id: PROTICdbO:0000335
						// name: X!TandemPipeline PAI
						this.writeCvParam("PROTICdbO:0000335",
								"" + hashProt.getPAI(), "PAI");

						// [Term]
						// id: PROTICdbO:0000337
						// name: protein group number
						String pappsoGroupingNumber = Utils
								.getPappsoGroupingNumber(group, sg, k);
						if (pappsoGroupingNumber != null) {
							this.writeCvParam("PROTICdbO:0000337",
									pappsoGroupingNumber, "grouping number");
						}

						writer.writeEndElement();// "proteinHit");
						k++;
					}
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						Protein prot2 = (Protein) hashProt.getGrpProtMatch()
								.getGrpProtein();
						// peptidesHitRef

						ArrayList<Peptide> peptideList = null;
						if (this.xtpExperimentType.equals("phospho")) {
							peptideList = ((Match) hashProt.getGrpProtMatch())
									.getValidPeptideInPhosphoIsland((HashSampleScanSetPhospho) hashProt);
						} else {
							peptideList = ((Match) hashProt.getGrpProtMatch())
									.get_valid_peptides();
						}

						for (Peptide pep : peptideList) {
							// for (Peptide pep : ((Match)
							// hashProt.getGrpProtMatch())
							// .get_peptide_order_inclus_redon()) {
							String xmlId = pepSampleScanCle2xmlId.get(pep
									.get_sample_scan_sequencepeptide_cle());
							if (xmlId == null) {
								String message = "the xml of peptide "
										+ pep.get_sample_scan_sequencepeptide_cle()
										+ " is not known in pepSampleScanCle2xmlId";
								logger.debug(message);
								throw new Exception(message);
							}
							writer.writeStartElement("peptideHitRef");
							writer.writeAttribute("peptide_hit_id", "" + xmlId);

							writer.writeStartElement("fromSeq");
							writer.writeAttribute(
									"seq_id",
									""
											+ prot_to_id.get(prot2
													.get_description()));
							writer.writeAttribute("start", "" + pep.get_start());
							writer.writeAttribute("stop", "" + pep.get_stop());
							String res1 = pep.get_pre();
							res1 = res1.replaceAll("]", "-");
							writer.writeAttribute("residue_before_nter", ""
									+ res1.charAt((res1.length() - 1)));
							String res2 = pep.get_post();
							res2 = res2.replaceAll("]", "-");
							writer.writeAttribute("residue_after_cter", ""
									+ res2.charAt(0));

							writer.writeEndElement();// fromSeq
							writer.writeEndElement();// peptideHitRef
						}

					}
					writer.writeEndElement();// "match");
				}
			}
			writer.writeEndElement();// "matchs");
		} catch (Exception e) {
			String message = "error writing Matchs in ProticDbMl \n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeCvParam(String acc, String value, String description)
			throws XMLStreamException {
		/**
		 * writer.writeStartElement("cvParam");
		 *
		 * writer.writeAttribute( "value", "" + ((Match) hashProt
		 * .getGrpProtMatch()) .get_coverage()); writer.writeAttribute("name",
		 * "Coverage determination by Benuch"); writer.writeAttribute("cvLabel",
		 * "PROTICdbO"); writer.writeAttribute("accession",
		 * "PROTICdbO:0000285"); writer.writeEndElement();// cvParam
		 */
		writer.writeStartElement("cvParam");

		writer.writeAttribute("value", value);
		if (description == null) {
			writer.writeAttribute("name", "N.A.");
		} else {
			writer.writeAttribute("name", description);

		}
		writer.writeAttribute("cvLabel", "PROTICdbO");
		writer.writeAttribute("accession", acc);
		writer.writeEndElement();// cvParam

	}
}
