/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.XtandemPipelineMain;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;

public class OdsGeneralInformationOutput {
	private static final Logger logger = Logger
			.getLogger(OdsGeneralInformationOutput.class);

	protected OdsDocWriterInterface odsTable;

	protected String sheetName = "X!TandemPipeline informations";

	protected OdsGeneralInformationOutput() {

	}

	public OdsGeneralInformationOutput(OutputStream outputStream)
			throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			// X!TandemPipeline informations
			odsTable.writeSheet(sheetName);
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsGeneralInformationOutput(OdsDocWriterInterface odsTable)
			throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet(sheetName);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing " + sheetName + "\n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void writeXtpSessionInformations(DataType_Base datatype,
			ArrayList<Identification> identificationList) throws MSMSException {
		try {

			odsTable.writeLine();
			odsTable.writeCell("X!TandemPipeline version :");
			odsTable.writeLine();
			odsTable.writeCell(XtandemPipelineMain.version);
			odsTable.writeCell(XtandemPipelineMain.versionName);
			odsTable.writeLine();
			odsTable.writeLine();
			odsTable.writeCell("X!TandemPipeline filters :");
			odsTable.writeLine();
			odsTable.writeCell("Mode");
			odsTable.writeCell(datatype.getType());
			odsTable.writeLine();
			odsTable.writeCell("Maximum peptide Evalue");
			odsTable.writeCell(datatype.getConf().get_peptide_evalue());
			odsTable.writeLine();
			odsTable.writeCell("Maximum protein Evalue");
			odsTable.writeCell(datatype.getConf().get_protein_evalue());
			odsTable.writeLine();
			odsTable.writeCell("Minimum number of peptides per protein");
			odsTable.writeCell(datatype.getConf().get_peptide_number());
			odsTable.writeLine();
			if (datatype.getType().equals("combi")
					|| datatype.getType().equals("phospho")) {
				if (datatype.getConf().get_protein_filter_to_complete()) {
					odsTable.writeCell("Protein filters are applied to all samples together");
				} else {
					odsTable.writeCell("Protein filters are applied within a sample");
				}

				odsTable.writeLine();
			}

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
