/*******************************************************************************
 * Copyright (c) 2016 Olivier Langella <Olivier.Langella@u-psud.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.sax_parse;

import org.apache.log4j.Logger;
import org.xml.sax.*;

import java.util.Hashtable;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.*;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.PeptideStore;

public class HandlerMzIdentMlResultFile extends SaxBaseHandler {
	protected final Logger logger = Logger.getLogger(this.getClass());

	private IdentificationDataSource sample_;

	protected Protein prot_;

	protected Peptide peptide;

	protected Identification identification_;

	// private Hashtable<Peptide, String> peptide2domainId = new
	// Hashtable<Peptide, String>();

	private Hashtable<String, String> correction_scan;

	private Hashtable<String, String> correction_time;

	protected Vector<Peptide> peptide_list;

	protected Match match_;

	private String _RT;

	private int _scan;

	private double _mhplus_obser;

	private String _evalue_prot;

	private int _charge;

	private String currentNoteLabel;

	private String currentGroupLabel;

	private String currentGroupType;

	private String currentNoteType;

	private Pattern pattBruckerRtScanNum = Pattern
			.compile("RTINSECONDS=([0-9]+.[0-9]+).*?MS: ([0-9]+)/ MSMS: ([0-9]+).*");

	private String precursorUnit;

	private double maxPrecursorRange;

	private double minPrecursorRange;

	private boolean _xtandem_missed_cleavage_bug_filter = false;

	public HandlerMzIdentMlResultFile(Identification L, IdentificationDataSource S) {
		super();
		if (L == null) {
			identification_ = new Identification();
		} else {
			identification_ = L;
		}
		sample_ = S;
	}

	@Override
	public void startDocument() throws SAXException {
		try {
			prot_ = new Protein();
			match_ = new Match();
			_scan = 0;
			_mhplus_obser = 0;
			_charge = 0;
			_evalue_prot = "";
			_RT = "";
			identification_.getMsRunSet().getInstance(sample_);
			correction_scan = new Hashtable<String, String>();
			correction_time = new Hashtable<String, String>();
			peptide_list = new Vector<Peptide>();
			logger.info("Parse Xtandem file in normal Mode : "
					+ sample_.getName());
		} catch (Exception e) {
			String message = "ERROR reading xtandem file :\n" + e.getMessage();
			logger.error(message);
			throw new SAXException(message);
		}
	}

	@Override
	public void endDocument() throws SAXException {
		// correction du numéro de scan
		// Enumeration<match> E =
		// identification_.get_protein_liste().elements();
		// while (E.hasMoreElements()) {
		// peptide[] peps = E.nextElement().get_peptide_order_inclus_redon();
		// for (int i = 0; i < peps.length; i++) {

		for (Peptide pep : peptide_list) {

			// if (pep.get_sample().equals(sample_.getName())) {
			if (correction_scan.containsKey("" + pep.get_scan())) {
				int scanNumInXml = pep.get_scan();
				pep.set_scan(Integer.parseInt(correction_scan.get(""
						+ pep.get_scan())));
				this.sample_
						.setScanCorrespondance(pep.get_scan(), scanNumInXml);

			}
			if (correction_time.containsKey("" + pep.get_scan()))
				pep.set_RT(correction_time.get("" + pep.get_scan()));
			// }
		}

	}

	private boolean checkPrecursorRange(Peptide peptide2check) {
		if (precursorUnit.equals("ppm")) {
			double min = peptide2check.get_mz_theo()
					- ((peptide2check.get_mz_theo() * this.minPrecursorRange) / Utils.onemillion);
			double max = peptide2check.get_mz_theo()
					+ ((peptide2check.get_mz_theo() * this.maxPrecursorRange) / Utils.onemillion);
			if ((peptide2check.getObservedMz() > min)
					&& (peptide2check.getObservedMz() < max)) {

				return true;
			}
			logger.warn("ppm z=" + peptide2check.getCharge() + " min=" + min
					+ " max=" + max + " mzobs=" + peptide2check.getObservedMz());

		} else if (precursorUnit.equals("Daltons")) {
			double min = peptide2check.get_mz_theo() - this.minPrecursorRange;
			double max = peptide2check.get_mz_theo() + this.maxPrecursorRange;

			if ((peptide2check.getObservedMz() > min)
					&& (peptide2check.getObservedMz() < max)) {
				return true;
			}
			logger.warn("daltons z=" + peptide2check.getCharge() + " min="
					+ min + " max=" + max + " mzobs="
					+ peptide2check.getObservedMz());
		}
		return false;
	}

	public void startElementprotein(String nameSpaceUri, Attributes attrs) {
		_evalue_prot = attrs.getValue("expect");
	}

	public void startElementfile(String nameSpaceUri, Attributes attrs) {
		if (attrs.getValue("type").equals("peptide"))
			prot_.setDatabase(identification_.getDatabaseSet().getInstance(
					attrs.getValue("URL")));
	}

	@Override
	protected void sendData(String data) {
		// public void endElementpeptide(String nameSpaceUri, String data) {
		if (checkPreviousElementName("peptide", 0)) {
			if (prot_.hasSequence() == false) {
				prot_.set_sequence(data);
			}
		}
	}

	public void startElementdomain(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		// <domain id="4017.1.1" start="21" end="31" expect="2.0e-06"
		// mh="1263.575"
		// delta="0.998" hyperscore="32.9" nextscore="12.2" y_score="10.7"
		// y_ions="9" b_score="0.0" b_ions="0"
		// pre="VLGR" post="VEFM" seq="TGSQGQCTQVR" missed_cleavages="10">

		// valeur généric du scan
		if (this.peptide == null) {
			this.peptide = PeptideStore.getInstance().newPeptide();
		}

		peptide.set_sequence(attrs.getValue("seq"));

		if (_xtandem_missed_cleavage_bug_filter) {
			int nb_missed_cleavages = Integer.parseInt(attrs
					.getValue("missed_cleavages"));
			if (peptide.getSequence().length() == (nb_missed_cleavages + 1)) {
				// this is the X!Tandem bug
				this.peptide = null;
				return;
			}
		}
		peptide.set_sample_id(identification_.getMsRunSet()
				.getInstance(sample_), sample_);

		peptide.set_scan(_scan);
		peptide.set_mhplus_obser(_mhplus_obser);
		peptide.set_charge(_charge);
		peptide.set_RT(_RT);

		// valeur specifique de la séquence matche
		peptide.set_start(Integer.parseInt(attrs.getValue("start")));
		peptide.set_stop(Integer.parseInt(attrs.getValue("end")));
		peptide.set_evalue(Float.valueOf(attrs.getValue("expect")));
		peptide.set_mhplus_theo(Double.valueOf(attrs.getValue("mh")));
		peptide.set_deltamass(Double.valueOf(attrs.getValue("delta")));
		peptide.set_hypercorr(Float.valueOf(attrs.getValue("hyperscore")));
		peptide.set_pre(attrs.getValue("pre"));
		peptide.set_post(attrs.getValue("post"));

		// logger.debug("attrs.getValue(id)" + attrs.getValue("id"));
		// peptide2domainId.put(peptide, "" + attrs.getValue("id"));
	}

	public void startElementaa(String nameSpaceUri, Attributes attrs) {
		if (this.peptide == null) {
			return;
		}
		peptide.set_Modifs(attrs.getValue("type"),
				Integer.parseInt(attrs.getValue("at")),
				Float.valueOf(attrs.getValue("modified")));
		// modifs liste for isotopic analysis
		identification_.add_modifs_to_liste(new Modifs(Float.valueOf(attrs
				.getValue("modified"))));
	}

	public void startElementgroup(String nameSpaceUri, Attributes attrs) {
		// logger.debug("startElementgroup begin");
		// <group label="performance parameters" type="parameters">
		this.currentGroupLabel = attrs.getValue("label");
		this.currentGroupType = attrs.getValue("type");
		if (currentGroupType.equals("model")) {
			_scan = Integer.parseInt(attrs.getValue("id"));
			_mhplus_obser = Double.valueOf(attrs.getValue("mh"));
			_charge = Integer.parseInt(attrs.getValue("z"));
			_RT = attrs.getValue("rt");
		}
	}

	public void endElementgroup(String nameSpaceUri, String data)
			throws MSMSException {
		// <group label="performance parameters" type="parameters">
		this.currentGroupLabel = null;
		this.currentGroupType = null;
	}

	public void startElementnote(String nameSpaceUri, Attributes attrs) {
		// logger.debug("startElementnote begin");
		// <note type="input" label="output, path hashing">no</note>
		this.currentNoteLabel = attrs.getValue("label");
		this.currentNoteType = attrs.getValue("type");
		// if (attrs.getValue("label").equals("description"))
		// prot_description = true;
		// if (attrs.getValue("label").equals("Description"))
		// spectre_number = true;
		// Ajouter la récupération du scan
	}

	public void endElementnote(String nameSpaceUri, String data)
			throws MSMSException {
		// <protein expect="-40.5" id="1947.2" uid="101553"
		// label="GRMZM2G119071_P01 P30756 Histone H2B.2 seq=translation; coord=4:63092571..63093826:-1;..."
		// sumI="7.95" >
		// <note label="description">GRMZM2G119071_P01 P30756 Histone H2B.2
		// seq=translation; coord=4:63092571..63093826:-1;
		// parent_transcript=GRMZM2G119071_T01; parent_gene=GRMZM2G119071</note>

		if (checkPreviousElementName("protein", 1)
				&& currentNoteLabel.equals("description")) {
			prot_.set_description(data);
			if (identification_.contain_protein(prot_)) {
				match_.set_protein_match(prot_);
				identification_.add_protein(prot_, match_);
			}

			else {
				match_ = (Match) identification_.get_match(prot_);
				prot_ = match_.get_protein_match();
			}
			prot_.set_evalue(
					identification_.getMsRunSet().getInstance(sample_),
					Float.valueOf(_evalue_prot));
		}

		if (checkPreviousElementName("group", 1)
				&& currentNoteLabel.equals("description")
				&& this.currentGroupLabel.equals("fragment ion mass spectrum")) {
			// <group type="support" label="fragment ion mass spectrum">
			// <note
			// label="Description">20120906_balliau_extract_1_A07_tca1-2.mzXML
			// scan 2012 (charge 3)</note>

			Matcher m = pattBruckerRtScanNum.matcher(data.replace("\n", "")
					.replace("\r", ""));
			if (m.matches()) {
				float rtime = Float.valueOf(m.group(1));
				int scan = Integer.parseInt(m.group(3));
				correction_scan.put("" + _scan, "" + scan);
				correction_time.put("" + scan, "" + rtime);
				logger.debug("Bruker MGF scan time fix " + scan + " = " + rtime);
			} else {
				String tmp = data.replaceAll(" .*", "");
				String[] split = tmp.split("-");
				if (split.length == 2) {
					try {
						int scan = Integer.parseInt(split[0].replaceAll(
								"ScanNumber:", ""));
						float rtime = Float.valueOf(
								split[1].replaceAll("RtInSec:", ""))
								.floatValue();
						correction_scan.put("" + _scan, "" + scan);
						correction_time.put("" + scan, "" + rtime);
						logger.debug("" + scan + " = " + rtime);
					} catch (Exception e) {
					}
				}
			}
		}

		if (this.currentGroupLabel.equals("input parameters")
				&& this.currentGroupType.equals("parameters")) {
			if (currentNoteType.equals("input")
					&& currentNoteLabel
							.equals("spectrum, parent monoisotopic mass error minus")) {
				// <group label="input parameters" type="parameters">
				// <note type="input"
				// label="spectrum, parent monoisotopic mass error minus">10</note>
				this.minPrecursorRange = Double.valueOf(data);
			}
			if (currentNoteType.equals("input")
					&& currentNoteLabel
							.equals("spectrum, parent monoisotopic mass error plus")) {
				// <note type="input"
				// label="spectrum, parent monoisotopic mass error plus">10</note>
				this.maxPrecursorRange = Double.valueOf(data);
			}
			if (currentNoteType.equals("input")
					&& currentNoteLabel
							.equals("spectrum, parent monoisotopic mass error units")) {
				// <note type="input"
				// label="spectrum, parent monoisotopic mass error units">ppm</note>
				this.precursorUnit = data;
			}
		}

		this.currentNoteLabel = null;
		this.currentNoteType = null;
	}

	public void endElementprotein(String nameSpaceUri, String data) {
		match_ = new Match();
		prot_ = new Protein();
	}

	public void endElementdomain(String nameSpaceUri, String data)
			throws MSMSException {
		if (this.peptide != null) {
			identification_.add_peptide(this.peptide);
			match_.add_peptide_match(peptide);
			peptide_list.add(peptide);
			peptide = null;
		}
	}

	public Identification get_identifications() {
		return (identification_);
	}

}
