package fr.inra.pappso.xtandempipeline.sax_parse;

/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>,
 * Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

import java.io.File;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MzxmlFile;

public class HandlerXtResultFile4ProcessingParams extends SaxBaseHandler {
	private static final Logger logger = Logger
			.getLogger(HandlerXtResultFile4ProcessingParams.class);
	private String xtandemVersion;
	private String currentGroupLabel = null;
	private String currentGroupType = null;
	private String currentNoteLabel = null;
	private String modelFileName;
	private String currentNoteType;
	private String biomlLabel;
	private int totalSpectraUsed;
	private int totalSpectraAssigned;
	private int totalUniqueAssigned;
	private ArrayList<String> databaseFileList = new ArrayList<String>();

	public HandlerXtResultFile4ProcessingParams() throws MSMSException {
	}

	@Override
	public void endDocument() throws SAXException {
	}

	public void startElementbioml(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		logger.debug("startElementbioml begin");
		/*
		 * <?xml version="1.0"?> <?xml-stylesheet type="text/xsl"
		 * href="tandem-style.xsl"?> <bioml
		 * xmlns:GAML="http://www.bioml.com/gaml/" label=
		 * "models from '/gorgone/pappso/moulon/raw/20120208_Blein_Rep4/20120208_Blein_rep4_1_A01_DU24-4-18-314.mzXML'"
		 * >
		 */
		biomlLabel = attrs.getValue("label");

	}

	public void startElementgroup(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		// logger.debug("startElementgroup begin");
		// <group label="performance parameters" type="parameters">
		this.currentGroupLabel = attrs.getValue("label");
		this.currentGroupType = attrs.getValue("type");

	}

	public void endElementgroup(String nameSpaceUri, String data)
			throws MSMSException {
		// <group label="performance parameters" type="parameters">
		this.currentGroupLabel = null;
		this.currentGroupType = null;
	}

	public void startElementnote(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		// logger.debug("startElementnote begin");
		// <note type="input" label="output, path hashing">no</note>
		this.currentNoteLabel = attrs.getValue("label");
		this.currentNoteType = attrs.getValue("type");

	}

	public void endElementnote(String nameSpaceUri, String data)
			throws MSMSException {
		// <group label="performance parameters" type="parameters">
		// <note label="process, version">x! tandem CYCLONE
		// (2011.12.01.1)</note>
		if ((currentGroupLabel.equals("performance parameters"))
				&& (this.currentGroupType.equals("parameters"))) {
			//<group label="performance parameters" type="parameters">
			//<note label="list path, sequence source #1">/gorgone/pappso/formation/Janvier2014/TD/Database/contaminants_standarts.fasta</note>
			//<note label="list path, sequence source #2">/gorgone/pappso/formation/Janvier2014/TD/Database/Genome_Z_mays_5a.fasta</note>
			if (this.currentNoteLabel.startsWith("list path, sequence source #")) {
				this.databaseFileList.add(data);
			}
			if (this.currentNoteLabel.equals("process, version")) {
				xtandemVersion = data;
			}
			// <note label="modelling, total spectra assigned">1</note>
			if (this.currentNoteLabel
					.equals("modelling, total spectra assigned")) {
				this.totalSpectraAssigned = new Integer(data);
			}
			// <note label="modelling, total spectra used">815</note>
			if (this.currentNoteLabel.equals("modelling, total spectra used")) {
				this.totalSpectraUsed = new Integer(data);
			}
			// <note label="modelling, total unique assigned">1</note>
			if (this.currentNoteLabel
					.equals("modelling, total unique assigned")) {
				this.totalUniqueAssigned = new Integer(data);
			}
		}
		if ((currentGroupLabel.equals("input parameters"))
				&& (this.currentGroupType.equals("parameters"))) {
			if (this.currentNoteLabel.equals("list path, default parameters")
					&& this.currentNoteType.equals("input")) {
				this.modelFileName = data;
			}
		}
		this.currentNoteLabel = null;
		this.currentNoteType = null;
	}

	public String getXtandemVersionNumber() {
		Pattern patt = Pattern.compile("X! TANDEM [A-Z]+ \\((.*)\\)",
				Pattern.CASE_INSENSITIVE);

		Matcher m = patt.matcher(xtandemVersion);
		String version = "";
		if (m.matches()) {
			version = m.group(1);
		}
		return version;
	}

	public String getXtModelFileName() throws Exception {
		logger.debug("getXtModelFileName " + this.modelFileName);
		// C:/Benperl/Xtandem/LCQ_analysis_from_CID.xml
		// "l"
		Pattern patt = Pattern.compile(".*?([^/\\\\]+)$");

		Matcher m = patt.matcher(this.modelFileName.trim());
		String version = "";
		if (m.matches()) {
			version = m.group(1);
		}
		return version;
	}

	public String getMSMSDataSourceFileName() {
		// "models from '/gorgone/pappso/moulon/raw/20120208_Blein_Rep4/20120208_Blein_rep4_1_A01_DU24-4-18-314.mzXML'"
		Pattern patt = Pattern.compile(".*?([^/\\\\]+)'$");

		Matcher m = patt.matcher(this.biomlLabel.trim());
		String version = "";
		if (m.matches()) {
			version = m.group(1);
		}
		return version;
	}

	public File getMSMSDataSourceFile() {
		// "models from '/gorgone/pappso/moulon/raw/20120208_Blein_Rep4/20120208_Blein_rep4_1_A01_DU24-4-18-314.mzXML'"
		Pattern patt = Pattern.compile("^models from '(.*)'$");

		File mzXmlFile = null;

		Matcher m = patt.matcher(this.biomlLabel.trim());
		String version = null;
		if (m.matches()) {
			version = m.group(1);
		}
		if (version != null) {
			logger.debug("filename getMSMSDataSourceFile " + this.biomlLabel
					+ " " + version);
			mzXmlFile = new File(version);
		}
		return mzXmlFile;
	}

	public MzxmlFile getMzxmlFile() {
		// "models from '/gorgone/pappso/moulon/raw/20120208_Blein_Rep4/20120208_Blein_rep4_1_A01_DU24-4-18-314.mzXML'"
		Pattern patt = Pattern.compile("^models from '(.*)'$");

		MzxmlFile mzXmlFile = null;

		Matcher m = patt.matcher(this.biomlLabel.trim());
		String version = null;
		if (m.matches()) {
			version = m.group(1);
		}
		if (version != null) {
			logger.debug("filename getMSMSDataSourceFile " + this.biomlLabel
					+ " " + version);
			mzXmlFile = new MzxmlFile(version);
		}
		return mzXmlFile;
	}

	public int getTotalSpectraUsed() {
		return totalSpectraUsed;
	}

	public int getTotalSpectraAssigned() {
		return totalSpectraAssigned;
	}

	public int getTotalUniqueAssigned() {
		return totalUniqueAssigned;
	}

	public ArrayList<String> getDatabaseFileList() {
		return databaseFileList;
	}

}
