package fr.inra.pappso.xtandempipeline.class_msms.MapDb;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.grouping.HashPeptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPeptide;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;

public class Peptide implements Comparable<Peptide>, GrpPeptide, Serializable {

	/**
	 *
	 */
	private static final long serialVersionUID = 5736723594583842634L;

	private static final Logger logger = Logger.getLogger(Peptide.class);

	protected transient IdentificationDataSource sample_file = null;

	protected transient HashSampleScan hashSampleScan = null;

	protected float RT;

	protected double mhplus_obser;

	protected double mhplus_theo;

	protected double deltamass;

	protected String sequence;

	protected String pre;

	protected String post;

	protected int start;

	protected int stop;

	protected int charge;

	protected float evalue;

	protected float hypercorr;

	protected ArrayList<Modifs> modifList;

	protected boolean isValid;

	protected ArrayList<Double> mz_values;

	protected ArrayList<Double> intensity_values;

	@Override
	public int compareTo(Peptide p) {
		int i;
		try {
			if (p == null) {
				throw new MSMSException("p is null");
			}
			i = this.getHashSampleScan().compareTo(p.getHashSampleScan());
		} catch (MSMSException e) {
			String message = e.getMessage();
			logger.error(message);
			i = 0;
		}
		if (i != 0)
			return (i);
		float delta = this.evalue - p.get_evalue();
		if (delta > 0) {
			return (1);
		} else if (delta < 0) {
			return (-1);
		} else {
			return (0);
		}
	}

	// constructeur
	public Peptide() {
		// logger.debug("Peptide constructor");
		this.modifList = new ArrayList<Modifs>();
		this.RT = 0;
		this.isValid = true;
		this.hashSampleScan = null;
	}

	public boolean isValid() {
		return isValid;
	}

	public void setIsValid(boolean validate) {
		this.isValid = validate;

		this.store();
	}

	public Peptide store() {
		return this;
	}

	public String get_sample_scan_sequencepeptide_cle() {
		String cle = this.get_sample() + "_" + this.get_scan() + "_"
				+ this.getSequence() + "_" + this.get_Modifs_formater();
		return (cle);
	}

	public void set_sample_id(MsRun msRun, IdentificationDataSource origin)
			throws MSMSException {
		this.sample_file = msRun.addFile(origin);
		if (this.hashSampleScan == null) {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(-1, msRun);
		} else {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(hashSampleScan.getScan(), msRun);
		}
	}

	public void set_scan(int i) {
		if (this.hashSampleScan == null) {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(i, null);
		} else {
			hashSampleScan = XtandemPipelineSession.getInstance()
					.getHashSampleScanInstance(i,
							hashSampleScan.getGrpMsSample());
		}
	}

	public void set_charge(int i) {
		this.charge = i;
	}

	public void set_RT(String i) {
		i = i.replaceAll("PT", "");
		i = i.replaceAll("S", "");
		try {
			this.RT = Float.valueOf(i).floatValue();
		} catch (Exception e) {
			this.RT = 0;
		}
	}

	public void set_mhplus_obser(double _mhplus_obser) {
		this.mhplus_obser = _mhplus_obser;
	}

	public void set_mhplus_theo(double i) {
		this.mhplus_theo = i;
	}

	public void set_pre(String i) {
		this.pre = i;
	}

	public void set_post(String i) {
		this.post = i;
	}

	public void set_start(int i) {
		this.start = i;
	}

	public void set_stop(int i) {
		this.stop = i;
	}

	public void set_deltamass(double f) {
		this.deltamass = f;
	}

	public void set_evalue(float f) {
		this.evalue = f;
	}

	public void set_hypercorr(float f) {
		this.hypercorr = f;
	}

	public void set_Modifs(String AA, int posi_in_prot, float modvalue) {
		int posi_corrected = posi_in_prot - this.start + 1;
		Modifs mod = new Modifs(AA, posi_corrected, modvalue);
		mod.setPosi_in_prot(posi_in_prot);
		this.modifList.add(mod);
		this.store();
	}

	public void set_sequence(String S) {
		this.sequence = S;
	}

	public String get_sample() {
		return (hashSampleScan.getGrpMsSample().getSampleName());
	}

	public int get_scan() {
		return (hashSampleScan.getScan());
	}

	public int getCharge() {
		return (charge);
	}

	public float getRetentionTimeInMinutes() {
		int temp = (int) (this.RT / 0.6);
		Float t = (float) temp / 100;
		return (t);
	}

	public int get_RT_in_sec() {
		return ((int) this.RT);
	}

	public double get_mhplus_obser() {
		return (this.mhplus_obser);
	}

	public double get_mhplus_theo() {
		return (this.mhplus_theo);
	}

	public String get_pre() {
		return (this.pre);
	}

	public String get_post() {
		return (this.post);
	}

	public int get_start() {
		return (this.start);
	}

	public int get_stop() {
		return (this.stop);
	}

	public double get_deltamass() {
		return (this.deltamass);
	}

	public float get_evalue() {
		return (this.evalue);
	}

	public float get_hypercorr() {
		return (this.hypercorr);
	}

	public ArrayList<Modifs> get_Modifs() {
		return (this.modifList);
	}

	public String get_Modifs_formater() {
		StringBuilder format = new StringBuilder();
		if (modifList.size() >= 1) {
			Modifs mod = modifList.get(0);
			format.append(mod.get_formater_value());
		}
		if (modifList.size() >= 2) {
			for (int i = 1; i < modifList.size(); i++) {
				Modifs mod = modifList.get(i);
				format.append(" - " + mod.get_formater_value());
			}
		}
		format.append(" ");
		return (format.toString());
	}

	public Double getMassShift(boolean ignoreIsotopeLabels, boolean removeMS3) {
		Double f = (double) 0;
		for (Modifs mod : modifList) {
			if (ignoreIsotopeLabels && mod.isIsotopicLabel()) {
				// ignoring this modification
			} else {
				if (mod.is_phopho_modifs() && removeMS3) {
					f += Utils.phospho_mass;
				} else {
					f += mod.get_modvalue();
				}
			}
		}
		return (f);
	}

	public Double get_mass_Modifs_without_isotope(boolean removeMS3) {
		Double f = (double) 0;
		for (Modifs mod : modifList) {
			if (!mod.isIsotopicLabel()) {
				if (mod.is_phopho_modifs() && removeMS3)
					f += Utils.phospho_mass;
				else
					f += mod.get_modvalue();
			}
		}
		return (f);
	}

	public Double get_mh_theo_without_isotope(boolean correctedMS3) {
		double f = this.mhplus_theo;
		for (Modifs mod : modifList) {
			if (mod.isIsotopicLabel()) {
				f -= mod.get_modvalue();
			} else if (mod.is_phopho_modifs() && correctedMS3) {
				f -= mod.get_modvalue();
				f += Utils.phospho_mass;
			}
		}
		return (f);
	}

	public boolean is_phopho_peptide() {
		boolean phospho = false;
		for (Modifs mod : modifList) {
			if (mod.is_phopho_modifs()) {
				// phospho = true;
				return (true);
			}
		}
		return (phospho);
	}

	@Override
	public Collection<GrpPhosphoModif> getGrpPhosphoModifs() {
		ArrayList<GrpPhosphoModif> phospho = new ArrayList<GrpPhosphoModif>();
		for (Modifs mod : modifList) {
			if (mod.is_phopho_modifs())
				phospho.add(mod);
		}
		return (phospho);
	}

	@Override
	public String getSequence() {
		return (sequence);
	}

	public String get_LI_sequence() {
		String seq = this.sequence.replaceAll("L", "I");
		return (seq);
	}

	public String get_LI_sequence_modifs() {
		String seq = this.get_LI_sequence();
		seq = seq + "-"
				+ this.get_mass_Modifs_without_isotope(false).intValue();
		return (seq);
	}

	public String get_sequence_formater() {
		String format = "" + this.pre + "." + this.sequence + "." + this.post;
		return (format);
	}

	public double get_mz_theo() {
		double mz = (double) ((this.get_mhplus_theo() + (((double) this
				.getCharge() - (double) 1) * Utils.mhplus)) / (double) this
				.getCharge());
		return mz;
	}

	public String get_phospho_Modifs_position() {
		Collection<GrpPhosphoModif> pho = this.getGrpPhosphoModifs();
		String S = "";
		for (GrpPhosphoModif posi : pho) {
			S += " " + posi.getPosi_in_prot();
		}
		S.replaceFirst(" ", "");
		return S;
	}

	public IdentificationDataSource getIdentificationDataSource()
			throws MSMSException {
		return sample_file;
	}

	public void setIntensity_values(String C_data, float factor) {
		C_data = C_data.replaceAll("\n", " ");
		C_data = C_data.replaceAll("^\\s+", "");
		C_data = C_data.replaceAll("\\s+$", "");
		String[] values = C_data.split(" ");
		this.intensity_values = new ArrayList<Double>();
		for (int i = 0; i < values.length; i++)
			intensity_values.add(Double.valueOf(values[i]).doubleValue()
					* factor);
	}

	public void setIntensity_values(double[] intenArr) {
		this.intensity_values = new ArrayList<Double>();
		for (int i = 0; i < intenArr.length; i++)
			intensity_values.add(Double.valueOf(intenArr[i]).doubleValue());
	}

	public ArrayList<Double> getIntensity_values() {
		return intensity_values;
	}

	public void setMz_values(String C_data) {
		C_data = C_data.replaceAll("\n", " ");
		C_data = C_data.replaceAll("^\\s+", "");
		C_data = C_data.replaceAll("\\s+$", "");
		String[] values = C_data.split(" ");
		this.mz_values = new ArrayList<Double>();
		for (int i = 0; i < values.length; i++)
			mz_values.add(Double.valueOf(values[i]).doubleValue());
	}

	public void setMz_values(double[] mzArr) {
		this.mz_values = new ArrayList<Double>();
		for (int i = 0; i < mzArr.length; i++)
			mz_values.add(Double.valueOf(mzArr[i]).doubleValue());
	}

	public ArrayList<Double> getMz_values() {
		return mz_values;
	}

	public MsRun getMsRun() {
		return (MsRun) hashSampleScan.getGrpMsSample();
	}

	public HashSampleScan getSampleScan() throws MSMSException {
		this.getHashSampleScan();
		if (hashSampleScan == null) {
			String error = "hashSampleScan is null";
			logger.debug(error);
			throw new MSMSException(error);
		}
		if (hashSampleScan.getGrpMsSample() == null) {
			return null;
		}
		if (hashSampleScan.getScan() == -1) {
			return null;
		}
		return hashSampleScan;
	}

	@Override
	public HashPeptide getHashPeptide() throws MSMSException {
		HashSampleScan hashScan = getSampleScan();
		if (hashScan == null) {
			throw new MSMSException("hashSampleScan is null for this peptide");
		}
		return XtandemPipelineSession.getInstance().getHashPeptideInstance(
				hashScan, this.get_LI_sequence());
	}

	@Override
	public boolean containsPosition(Integer position) throws Exception {
		if (position == null) {
			String error = "position is null";
			logger.debug(error);
			throw new Exception(error);
		}
		if (this.start > position) {
			return false;
		}
		if (position > this.stop) {
			return false;
		}
		return true;
	}

	@Override
	public HashSampleScan getHashSampleScan() throws MSMSException {
		return hashSampleScan;
	}

	public String get_Modifs_formater_without_isotope() {
		StringBuilder format = new StringBuilder();
		for (Modifs mod : modifList) {
			if (!mod.isIsotopicLabel()) {
				format.append(" " + mod.get_formater_value());
			}
		}
		format.append(" ");
		return (format.toString());
	}

	public double getMzDelta() {
		return (this.get_mz_theo() - this.getObservedMz());
	}

	public double getObservedMz() {
		double mz = (double) ((this.get_mhplus_obser() + (((double) this
				.getCharge() - (double) 1) * Utils.mhplus)) / (double) this
				.getCharge());
		return mz;
	}

	public void setRt(float floatValue) {
		this.RT = floatValue;
	}

	public void writeXpip(XMLStreamWriter writer) throws MSMSException {
		try {
			writer.writeStartElement("peptide");
			writer.writeAttribute("sample", this.get_sample());
			IdentificationDataSource dataSource = this
					.getIdentificationDataSource();
			dataSource.writeXpipPeptideSourceAttribute(writer);
			writer.writeAttribute("scan", "" + this.get_scan());

			Integer scanCorrespondance = dataSource.getScanCorrespondance(this
					.get_scan());
			if (scanCorrespondance != null) {
				writer.writeAttribute("scan_in_xtandem", ""
						+ scanCorrespondance);
			}
			writer.writeAttribute("RT", "" + this.get_RT_in_sec());
			writer.writeAttribute("mhplus_obser", "" + this.get_mhplus_obser());
			writer.writeAttribute("mhplus_theo", "" + this.get_mhplus_theo());
			writer.writeAttribute("deltamass", "" + this.get_deltamass());
			writer.writeAttribute("sequence", this.getSequence());
			writer.writeAttribute("pre", this.get_pre());
			writer.writeAttribute("post", this.get_post());
			writer.writeAttribute("start", "" + this.get_start());
			writer.writeAttribute("stop", "" + this.get_stop());
			writer.writeAttribute("charge", "" + this.getCharge());
			writer.writeAttribute("evalue", "" + this.get_evalue());
			writer.writeAttribute("hypercorr", "" + this.get_hypercorr());
			writer.writeAttribute("validate", "" + this.isValid());

			writer.writeStartElement("modifs");
			for (Modifs mod : this.get_Modifs()) {
				writer.writeEmptyElement("modif");
				writer.writeAttribute("aa", "" + mod.get_AA());
				writer.writeAttribute("modvalue", "" + mod.get_modvalue());
				writer.writeAttribute("posi", "" + mod.get_posi());
				writer.writeAttribute("posi_in_prot",
						"" + mod.getPosi_in_prot());
			}
			writer.writeEndElement();

			this.xpipWriteSpecificInfos(writer);

			// on ferme le peptide
			writer.writeEndElement();
		} catch (XMLStreamException e) {
			String message = "XMLStreamException ERROR writing XML in writeXpip "
					+ this.getClass() + "\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		} catch (MSMSException e) {
			String message = "ERROR in writeXpip " + this.getClass() + "\n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		} catch (Exception e) {
			String message = "Exception ERROR in writeXpip " + this.getClass()
					+ "\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		}

	}

	protected void xpipWriteSpecificInfos(XMLStreamWriter writer)
			throws MSMSException {

	}

	@Override
	public String getSequenceLiModifs() {
		return this.get_LI_sequence_modifs();
	}
}
