package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.Hashtable;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;

public class calculation_peptide_fdr extends calculation_base_protein {

	private String[] reverse_database;
	private config conf;
	private float reverse;
	private float normal;
	private Hashtable<HashSampleScan, String> peptide_view;

	public calculation_peptide_fdr(config conf, String[] databases) {
		super();
		this.reverse_database = databases;
		this.conf = conf;
		this.reverse = 0;
		this.normal = 0;
		this.peptide_view = new Hashtable<HashSampleScan, String>();
		logger.info("Calculation of peptide FDR using separated reverse database");
	}

	public calculation_peptide_fdr(config conf) {
		super();
		this.reverse_database = null;
		this.conf = conf;
		this.reverse = 0;
		this.normal = 0;
		this.peptide_view = new Hashtable<HashSampleScan, String>();
		logger.info("Calculation of peptide FDR using X!Tandem reverse option");
	}

	protected void processing_match(Match m) throws MSMSException {
		boolean rev = false;
		if (reverse_database != null) {
			for (String d : reverse_database) {
				if (m.get_protein_match().getDatabase().getDatabasePath()
						.equals(d)) {
					rev = true;
				}
			}
		} else {
			if (m.get_protein_match().isReversed())
				rev = true;
		}

		for (Peptide pep : m.get_peptide_order()) {
			boolean is_new = true;
			if (peptide_view.containsKey(pep.getHashSampleScan()))
				is_new = false;
			else
				peptide_view.put(pep.getHashSampleScan(), "ok");

			if (pep.get_evalue() <= conf.get_peptide_evalue() && is_new) {
				if (rev) {
					reverse++;
				} else {
					normal++;
				}
			}
		}
	}

	public float get_peptide_fdr() {
		logger.debug("normal :" + normal);
		logger.debug("reverse :" + reverse);

		if (normal == 0)
			return (0);
		float val = reverse / normal * 100;
		return (val);
	}
}
