package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.HashMap;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;

public class calculation_protein_fdr extends calculation_base_protein {

	private String[] reverse_database;
	private config conf;
	private float reverse;
	private float normal;
	private HashMap<HashSampleScan, String> peptide_view;

	public calculation_protein_fdr(config conf, String[] databases) {
		super();
		this.reverse_database = databases;
		this.conf = conf;
		this.reverse = 0;
		this.normal = 0;
		this.peptide_view = new HashMap<HashSampleScan, String>();
		logger.info("Calculation of protein fdr using separated reverse database");
	}

	public calculation_protein_fdr(config conf) {
		super();
		this.reverse_database = null;
		this.conf = conf;
		this.reverse = 0;
		this.normal = 0;
		this.peptide_view = new HashMap<HashSampleScan, String>();
		logger.info("Calculation of protein fdr using X!Tandem/Mascot reverse option");
	}

	protected void processing_match(Match m) throws MSMSException {
		// check that protein match is totally new :
		// if there is at least one peptide that was not seen before
		boolean specific_peptide = true;
		for (Peptide pep : m.get_peptide_order()) {
			if (peptide_view.containsKey(pep.getHashSampleScan())) {
				specific_peptide = false;
			}
		}

		// if the match is valid :
		// 1) contains sufficient unique valid peptide (2 pep by prot for
		// example)
		// 2) the protein evalue is above the defined threshold
		// 3) the match contains at least one specific peptide
		if (m.get_unique_valid_peptide_number() >= conf.get_peptide_number()
				&& m.get_protein_match().get_evalue() <= conf
						.get_protein_evalue() && specific_peptide) {
			boolean rev = false;
			if (reverse_database != null) {
				for (String d : reverse_database) {
					if (m.get_protein_match().getDatabase().getDatabasePath()
							.equals(d)) {
						// the protein is identified from one reverse database
						rev = true;
					}
				}
			} else {
				if (m.get_protein_match().isReversed()) {
					// the protein match was set to be a reverse identification
					rev = true;
				}
			}
			if (rev)
				reverse++;
			else
				normal++;
		}
		for (Peptide pep : m.get_peptide_order()) {
			// mark all peptides as already seen
			peptide_view.put(pep.getHashSampleScan(), "ok");
		}
	}

	public float get_protein_fdr() {
		if (normal == 0)
			return (0);
		float val = reverse / normal * 100;
		return (val);
	}
}
