/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.input.fasta;

import java.io.InputStream;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;

public class FastaReaderProteinList implements FastaReader {
	private static final Logger logger = Logger
			.getLogger(FastaReaderProteinList.class);
	FastaInputStreamParser parser;
	private Identification identification;

	public FastaReaderProteinList(Identification identification) {
		this.identification = identification;
	}

	public void parseFastaInputStream(InputStream inputStream) throws Exception {
		parser = new FastaInputStreamParser(this);
		parser.parseFastaInputStream(inputStream);
	}

	public void newFastaSequence(String header, String strSequence)
			throws Exception {
		// logger.debug("newFastaSequence begin " + header);
		Protein protein = new Protein();
		protein.set_description(header);
		if (identification.contain_protein(protein)) {
			// new protein is not possible
			// logger.debug("newFastaSequence acc not found "
			// + protein.getAccession());
		} else {
			// get existing match for this protein
			Match match = identification.get_match(protein);
			match.get_protein_match().set_sequence(strSequence);
			match.tryToDeterminePeptidePositions();
		}
	}
}
